//==============================================================================
//
//    cr_infra_2m_p_ns_.c
//
//   AUTOR: CLAUDIO LRIOS
//   DATA: 24/03/2014
//   RECEPTOR DE 2 CANAIS (ABRE/FECHA)- P/ CONTROLE REMOTO  INFRAVERMELHO
//
//   MICROCONTROLADOR : PIC 12F675 - SENSOR TSOP1550
//
//   Obs.Somente para protocolo 'NEC' (LG,Semp,Elsys,etc) e opo para 'Sony'
//   timer 0 controla as piscada em modo de programao e o timer 1 conta
//   o tempo na recepo dos sinais infravermelhos oriundos do controle remoto.
//   Objetivo desta montagem:
//   Controlar 2 motores com reteno, para abrir /fechar persianas
//
//==============================================================================
//
//   Como programar incialmente:
//   1) Aperte o boto 'Prog' por mais que 2 segundos. Acender o led  'prog'.
//
//   2) Aperte uma tecla do controle, da sua escolha, que deseje gravar na
//   Eeprom. Ao ser aceito, piscar o led 'prog' uma vez.
//   Repita com mais 3 botes. O led 'prog' piscar  conforme a sequencia da
//   tecla programada. No quarto boto aprendido, sair do modo de programao.
//   Aperte o boto 'Prog' por 2 segundos para sair do modo de programao, se
//   desejar sair de forma antecipada.
//
//   Use controle universal rca sistmelink4 codigo '006' tv para nec e '002' tv
//   para  Sony.
//==============================================================================
//   Obs. Este dispositivo  apenas didtico, sendo montado apenas em protoboard
//   e sendo sujeito a bugs no identificados.
//
//==============================================================================
//Descomente o abaixo SOMENTE se perdeu o byte de calibrao

//   #define PERDI_BYTE_CALIBRACAO
//==============================================================================

#define  padrao_sony// padro 'sony' do controle remoto 
//#define  padrao_nec// padro 'nec' do controle remoto

#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(A)  //  sentido das portas feita pelo programador
#ifdef  PERDI_BYTE_CALIBRACAO  // calibrao do oscilador interno do pic
#ROM  0X3FF = {0X34fc}//mxima freq= 0x34fc ;mdia=0x3480; minima=0x3400
#endif
#ZERO_RAM

//==============================================================================
//   REGISTRADORES
//==============================================================================
#byte gpio   = 0x05
#byte tmr0   = 0x01
#byte tris   = 0x85
#byte t1con   =0x10
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte pir1    =0x0c
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte intcon  =0x8b
#byte status  =0x03
//==============================================================================
//   DEFINIO DE PINOS E SAIDAS
//==============================================================================


#bit  s2    = 0x5.2 //pino 5
#bit  s1    = 0x5.1 //pino 6
#bit  _prog = 0x5.0 //pino 7
#bit  _in   = 0x5.3 //pino 4
#bit  s3    = 0x5.4 //pino 3
#bit  s4    = 0x5.5 //pino 2

#bit  tris_prog = 0x85.0 // p/pino 7
#bit  tris_s1   = 0x85.1
#bit  tris_s2   = 0x85.2
#bit  tris_s3   = 0x85.4
#bit  tris_s4   = 0x85.5



#bit  cy     = 0x3.0 //carry bit
#bit  on_tmr1 = 0x10.0  //on/off tmr1
#bit  fov1    = 0x0c.0//flag over flow do tmr1
#bit  tmr1if  =  0x0c.0 //flag over flow do tmr1
#bit  tmr0if  = 0x8b.2//flag over tmr0
#define in  pin_a3


//==============================================================================
//   CONSTANTES GERAIS
//==============================================================================
#ifdef padrao_nec
int const qbits = 32;  //quantidade de bits a receber 'nec'
#endif
#ifdef padrao_sony
int const qbits = 12;  //quantidade de bits a receber 'sony'
#endif
int const   TINITPROG=150; // tempo mnimo para iniciar 'modo de programao'
int16 const  T_rev=1000; //tempo parado, ao reverter com motor que estava ligado
//==============================================================================
//   VARIVEIS GLOBAIS
//==============================================================================

int qb,passo,ax1,cntp1;
int buffer[4];
short flag_prog,flag_rok;


//==============================================================================
//   SUBROTINAS E FUNES GERAIS
//==============================================================================


//==============================================================================
//   GRAVA TECLA NA EEPROM
//==============================================================================
void grava( int a,int b){
 disable_interrupts(GLOBAL);
 write_eeprom(a,b);
 enable_interrupts(GLOBAL);}

//==============================================================================
//   COMPARA TECLA RECEBIDA COM TECLAS GRAVADAS NA EEPROM
//==============================================================================

short mem(int a){
#ifdef padrao_nec
 return ((buffer[0]== read_eeprom(a))&&(buffer[2]== read_eeprom((a+1))));}
#endif
#ifdef padrao_sony
return ((buffer[0]== read_eeprom(a))&&(buffer[1]== read_eeprom((a+1))));}
#endif



 //==============================================================================
//   INTERRUPO POR MUDANA DE ESTADO - RECEPO DO CONTROLE IR (PADRO 'NEC')
//==============================================================================
#ifdef  padrao_nec

#int_RA
void rx_nec_mudanca_gp3() {
 int16 const LIMITE_BYTE_LOW=1200, PULSO_9MS=7000,PULSO_4MS=3800;
 static int16 ref,gt;
 static short ov1,flag_p;


    gpio|=gpio; //ler port para conseguir zera gpif (DS41190C-page 21)
       if(!flag_rok){
           on_tmr1=0; ov1=tmr1if; gt=get_timer1(); set_timer1(0);
           tmr1if=0;  on_tmr1=1;//desliga timer1/pega valor/zera e religa


           if(!input(in)){         //input='0'
           if((gt>PULSO_9MS)||(ov1)){ qb=qbits;}

             if(flag_p){
              shift_right(&buffer[0],4,(gt>ref));
               if(--qb==0) { qb=qbits;

 //  if (!(buffer[0]+buffer[1]+buffer[2]+buffer[3]+0x2)){ flag_rok=1;}//normal
      if(!(buffer[2]+buffer[3]+1)){ flag_rok=1;}  //  extendido
                }
               }
            }
            else{                   //input='1'
                 if((gt>PULSO_4MS)||(ov1)){ qb=qbits; flag_p=0;}
                 else {
                 if((gt<LIMITE_BYTE_LOW)&& (!ov1)&& (!flag_p)){
                    flag_p=1; qb=qbits; ref=gt; ref<<=1; }

                }
            }

         }//if(!flag_rok)
}//void mudanca_gp3()

#endif

#ifdef   padrao_sony
//==============================================================================
//   INTERRUPO POR MUDANA DE ESTADO - RECEPO DO CONTROLE IR (PADRO 'SONY')
//==============================================================================
#int_RA
void rx_nec_mudanca_gp3() {
 int16 const  PULSO_2M4S=2000;
 static int16 ref,gt;
 static short ov1;


    gpio|=gpio; //ler port para conseguir zera gpif (DS41190C-page 21)
       if(!flag_rok){
           on_tmr1=0; ov1=tmr1if; gt=get_timer1(); set_timer1(0);
           tmr1if=0;  on_tmr1=1;//desliga timer1/pega valor/zera e religa


           if(input(in)){         //input='1'
           if((gt>PULSO_2M4S)||(ov1)){ qb=qbits;
            ref=gt; ref>>=2;ref+=ref>>1;    //obtem 900us
            buffer[0]=0;buffer[1]=0;
           }
               else {
                    shift_right(&buffer[0],2,(gt>ref));
                    if(--qb==0) { qb=qbits;
                                flag_rok=1;
                                //grcnt=TIME_RX_ON;
                                }
               }
            }
            else{                   //input='0'
                 if((gt>PULSO_2M4S)||(ov1)){ qb=qbits; }

            }

         }//if(!flag_rok)
}//void mudanca_gp3()

#endif


//==============================================================================
//
//                PROGRAMA PRINCIPAL
//
//==============================================================================

void main() {

    //==============================================================================
//   INICIALISAO DE PORTAS E REGISTRADORES
//==============================================================================

   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_timer_1 ( T1_INTERNAL | T1_DIV_BY_1 );
   gpio=0;
   ax1=0;
   tris=0b001001; //tudo sada exceto GP3/GP0
   set_timer1(0); //zera tmr1
   tmr1if=0 ; //apaga flag over flow
   on_tmr1=1;//liga timer1
   setup_counters(RTCC_INTERNAL,RTCC_DIV_256);
   enable_interrupts(INT_RA3);//habilita interrupo por mudana de estado
   enable_interrupts(global);
   delay_ms(20);
   cntp1=TINITPROG;
   qb=qbits;//carrega quantidade de bits a receber
   passo=0;//  bit_set(t1con,0);
   flag_rok=0;

//==============================================================================
//                         LOOP PRINCIPAL
//==============================================================================
   while(true){
      delay_ms(10);   //atraso temporizao


//==============================================================================
//   Desligamento de sadas pelo fim de curso
//==============================================================================


     if(!flag_prog){

     if(tris_s1){ if(s1==1) s2=0;}//desliga saidas
     if(tris_s2){ if(s2==1) s1=0;}
     if(tris_s3){ if(s3==1) s4=0;}
     if(tris_s4){ if(s4==1) s3=0;}

     }
//=============================================================================
//   TESTA BOTO 'PROG'  (INICIO/FIM DE MODO DE PROGRAMAO)
//==============================================================================
     tris_prog=1;
     if(!_prog) {
       if(--cntp1==0)  {
         cntp1=TINITPROG;
         if(flag_prog){ flag_prog=0;
         }
         else { flag_prog=1; passo=0;}
         gpio=0;
         }
     }
     else
     { cntp1=TINITPROG;}

 //==============================================================================
//   PROGRAMAR TECLAS DO  CONTROLE REMOTO NA EEPROM -MODO DE PROGRAMAO
//==============================================================================
     if(flag_prog){
     int b;
     _prog=1;//liga led
     tris_prog=0;//saida em prog


       switch (passo){
       case 0:

         if(flag_rok){
         grava(0,buffer[0]);
         #ifdef padrao_nec
         grava(1,buffer[2]);
         #endif
         #ifdef padrao_sony
         grava(1,buffer[1]);
         #endif
         _prog=0;
         delay_ms(250);//pisca led 'prog' 1x
         _prog=1;
         passo=2;}
         break;

       case 2:

         if(flag_rok){
         grava(2,buffer[0]);
         #ifdef padrao_nec
         grava(3,buffer[2]);
         #endif
         #ifdef padrao_sony
         grava(3,buffer[1]);
         #endif

         for(b=0;b<2;++b){//pisca led 'prog' 2x
          _prog=0;
         delay_ms(250);
         _prog=1;
          delay_ms(250);
         }
         passo=4;
         }
         break;

       case 4:

         if(flag_rok){
         grava(4,buffer[0]);
          #ifdef padrao_nec
         grava(5,buffer[2]);
         #endif
         #ifdef padrao_sony
         grava(5,buffer[1]);
         #endif

         for(b=0;b<3;++b){//pisca led 'prog' 3x
          _prog=0;
         delay_ms(250);
         _prog=1;
          delay_ms(250);
         }
         passo=6;
         }
         break;

       case 6:

         if(flag_rok){
         grava(6,buffer[0]);
         #ifdef padrao_nec
         grava(7,buffer[2]);
         #endif
         #ifdef padrao_sony
         grava(7,buffer[1]);
         #endif

         for(b=0;b<4;++b){//pisca led 'prog' 4x
          _prog=0;
         delay_ms(250);
         _prog=1;
          delay_ms(250);
         }
         passo=8;}
         break;

       case 8:

         if(flag_rok){
         grava(8,buffer[0]);
         #ifdef padrao_nec
         grava(9,buffer[2]);
         #endif
         #ifdef padrao_sony
         grava(9,buffer[1]);
         #endif


         }
         flag_prog=0; //sai do modo de programao
          break;

       }//switch passo
       flag_rok=0;
     }//if(flag_prog
     else { _prog=0; tris_prog=0;}//apaga led prog

 //=============================================================================
//   ACIONAR SADAS
//==============================================================================

 if(!flag_prog&&flag_rok){
 int a;
  if(mem(0)){
   if(!tris_s2&&s2){tris_s2=1; delay_ms(T_rev);}
   tris_s2=1;
   tris_s1=0;
   if(!s1)s1=1;
   else s1=0;
    }

   else
 if(mem(2)){
  if(!tris_s1&&s1){tris_s1=1; delay_ms(T_rev);}
   tris_s1=1;
   tris_s2=0;
   if(!s2) s2=1;// segundo toque desliga
   else s2=0;
  }
   else
 if(mem(4)){
  if(!tris_s3&&s3){tris_s3=1; delay_ms(T_rev);}
   tris_s3=1;
   tris_s4=0;
   if(!s4) s4=1;
   else s4=0;
 }
   else
 if(mem(6)){
   if(!tris_s4&&s4){tris_s4=1; delay_ms(T_rev);}
   tris_s4=1;
   tris_s3=0;
   if(!s3) s3=1;
   else s3=0;
 }
    #ifdef padrao_sony  //delay 
    for(a=0;a<150;++a){
     if(tris_s1){ if(s1==1) s2=0;}//desliga saidas
     if(tris_s2){ if(s2==1) s1=0;}
     if(tris_s3){ if(s3==1) s4=0;}
     if(tris_s4){ if(s4==1) s3=0;}
     delay_ms(2);
    }
    #endif
    flag_rok=0;
  }
 }//void while(true)
}//void main



